/*
 * Software fr ATMega8 und kompatible
 * fr PWM-Board turi
 * Basissoftware ausgelegt fr:
 *    RC5-Fernbedienung (Port D2)
 *    3 PWM-Kanle mit Helligkeitsregelung fr 10-Bit PWM in 64 Stufen
 *    4 ADC-Kanal fr Poti zur parallelen Helligkeitsregelung beider Kanle
 *    Test RGB-Fader analog mit Poti + Helligkeit
 * mit Board erweiterbar bis auf 4 ADC-Kanle und 3 PWM
 * Software-Komponenten verschiedener Foren verwendet und teilweise adaptiert (s. Doku
 * der jeweiligen Files)
 *
 */


// Komponenten fr PWM, RC5 und ADC 
#include "PWMTable.c"
#include "RC5_v1.c"
#include "adc.c"
//#include "taster.c"

#include <avr/eeprom.h> 
#include <util/delay.h>


uint16_t ee_pwm1 EEMEM = 42; 
uint16_t ee_pwm2 EEMEM = 44; 
uint16_t ee_pwm3 EEMEM = 46; 
uint16_t ee_pwm4 EEMEM = 48;
uint8_t ee_pwm5 EEMEM = 50;
 

int main (void) {            
 
   	DDRB  = 0xff;             
   	PORTB = 0b00001111;            
   	PORTB = 0x07;            

   	uint16_t ADCval, ADCvalC1, ADCvalC2, ADCvalC3, ADCvalC0 ;		//Werte des ADC
	uint8_t code, addr;			//RC5-Code, Adresse
	int ADCdiff=0;

	int16_t hue, sat, bri, red, blue, green, red_val, green_val, blue_val;

	uint8_t key=0;		//Taste fr Farbwahl (1,2,3); Taste 5 zum Einleiten Speichern
	uint8_t iProg=0;	//Programmart (0/9,6)

	//0 - Standard RGB mit Helligkeit
	//6 - HSB mit Helligkeit
	//9 - RGB ohne Helligkeit


	//10 Bit PWM mit 64 Stufen initialisieren
	pwm_10_64init();
	//Fernbedinung aktivieren
	rc5_init (RC5_ALL);

	/* Interrupts zulassen */
	sei();


	// Kanal 0,1 ADC einmalig lesen, um Startwerte zu lesen
	ADCval = ReadChannel(3);ADCvalC3=ADCval; /* MUX-Bits auf 0b0000 -> Channel 0 */
	ADCval = ReadChannel(2);ADCvalC2=ADCval; /* MUX-Bits auf 0b0000 -> Channel 0 */
	ADCval = ReadChannel(1);ADCvalC1=ADCval; /* MUX-Bits auf 0b0000 -> Channel 0 */
	ADCval = ReadChannel(0);ADCvalC0=ADCval; /* MUX-Bits auf 0b0000 -> Channel 1 */
	//EEPROM lesen und Wert setzen
	red_val=eeprom_read_word(&ee_pwm1);
	green_val=eeprom_read_word(&ee_pwm2);
	blue_val=eeprom_read_word(&ee_pwm3);
	bri=eeprom_read_word(&ee_pwm4);
	iProg=eeprom_read_byte(&ee_pwm5);

	//Anfangswert setzen
	pwm_10_64(red_val, 1); //PWM-Kanal 1
	pwm_10_64(green_val, 2); //PWM-Kanal 2
	pwm_10_64(blue_val, 3); //PWM-Kanal 3	
	//pwm_10_64(22, 1); //PWM-Kanal 1
	//pwm_10_64(10, 2); //PWM-Kanal 2
	//pwm_10_64(10, 3); //PWM-Kanal 3	
	//_delay_ms(2000);

	//Hauptschleife
   	while(1) {
			                
		   if (-1 == rc5.flip) //keine FB - Potiabfrage
		   {
		   		//keine FB-Taste gedrckt: Kanal 0,1 ADC lesen
				ADCval = ReadChannel(3); 
				//Differenz zum vorhergehenden Wert? -> Poti wurde verndert
				if(ADCvalC3<20) ADCdiff=ADCvalC3; else ADCdiff=20;
				if ((ADCval > ADCvalC3+20) || (ADCval < ADCvalC3-ADCdiff))
		 		{ 
					//Helligkeit gesamt
					ADCvalC3=ADCval;
					bri=ADCvalC3/16;  //0..64
				}				
				ADCval = ReadChannel(2); /* MUX-Bits auf 0b0000 -> Channel 1 */
				//Differenz zum vorhergehenden Wert? -> Poti wurde verndert
				if(ADCvalC2<20) ADCdiff=ADCvalC2; else ADCdiff=20;
				if ((ADCval > ADCvalC2+20) || (ADCval < ADCvalC2-ADCdiff))
				{
					//ADC-Wert speichern
					ADCvalC2=ADCval;
					blue_val=ADCvalC2/16; //0..64
				}
				ADCval = ReadChannel(1); /* MUX-Bits auf 0b0000 -> Channel 1 */
				//Differenz zum vorhergehenden Wert? -> Poti wurde verndert
				if(ADCvalC1<20) ADCdiff=ADCvalC1; else ADCdiff=20;
				if ((ADCval > ADCvalC1+20) || (ADCval < ADCvalC1-ADCdiff))
				{
					//ADC-Wert speichern
					ADCvalC1=ADCval;
					green_val=ADCvalC1/16; //0..64
					hue = ADCvalC1/4*3; //0..765
				}
				ADCval = ReadChannel(0); /* MUX-Bits auf 0b0000 -> Channel 0 */
				//Differenz zum vorhergehenden Wert? -> Poti wurde verndert
				if(ADCvalC0<20) ADCdiff=ADCvalC0; else ADCdiff=20;
				if ((ADCval > ADCvalC0+20) || (ADCval < ADCvalC0-ADCdiff))
				{
					//ADC-Wert speichern
					ADCvalC0=ADCval;
					red_val=ADCvalC0/16; //0..64
					sat = ADCvalC0/8;  //0..128
				}
		   }  //keine FB - Potiabfrage
		   else
		   {	//FB verarbeiten
		      	/* Ja, dann rc5.code merken und evtl. rc5.addr */
		      	code = rc5.code;
		      	addr = rc5.addr;
		      	/* und auf naechstes Zeichen warten */
		      	rc5.flip = -1;
			  if (addr==0)		//TV-Modus
			  {
				  switch (code)
				  {
					case 12: { 	red_val=0;green_val=0;blue_val=0;
								break;}    // Aus
		    		case 43: { 	red_val=15;green_val=15;blue_val=15;
								break;}    // Play
		    		case 44: { 	red_val=30;green_val=30;blue_val=30;
								break;}    // Stop
		    		case 53: { 	red_val=40;green_val=40;blue_val=40;
								break;}    // Pause
		    		case 13: { 	red_val=63;green_val=63;blue_val=63;
								break;}    // Mute
		    		case 60: { 	red_val=63;green_val=0;blue_val=0;
								break;}    // Record
		    		case 46: { 	red_val=0;green_val=63;blue_val=0;
								break;}    // back
		    		case 42: { 	if (key==5)
								{	//Speichern der akt. Werte eingeleitet
									eeprom_write_word(&ee_pwm1,red_val ); //akt. Helligkeit speichern
									eeprom_write_word(&ee_pwm2,green_val ); //akt. Helligkeit speichern
									eeprom_write_word(&ee_pwm3,blue_val ); //akt. Helligkeit speichern
									eeprom_write_word(&ee_pwm4,bri ); //akt. Helligkeit speichern
									eeprom_write_byte(&ee_pwm5,iProg ); //Programm speichern
									//eeprom_busy_wait();
									//Besttigung kurz blinken
									pwm_10_64(20, 1);_delay_ms(100);
									pwm_10_64(60, 1);_delay_ms(100);
									pwm_10_64(20, 1);_delay_ms(100);
									pwm_10_64(60, 1);_delay_ms(100);
									pwm_10_64(red_val, 1);
									key=0;
								}
								else
								{	red_val=0;green_val=0;blue_val=63;}
								break;}    // forward

		    		case 16:// Vol + in Abhngigkeit Zifferntaste
					 		{ 
								if (key==1)
								{
									red_val+=2;
									if (red_val > 63) red_val = 63;
								}
								if (key==2)
								{
									green_val+=2;
									if (green_val > 63) green_val = 63;
								}
								if (key==3)
								{
									blue_val+=2;
									if (blue_val > 63) blue_val = 63;
								}
								break;
							}    
		    		case 17:// Vol - in Abhngigkeit Zifferntaste
							 { 
								if (key==1)
								{
									red_val-=2;
									if (red_val < 0) red_val = 0;
								}
								if (key==2)
								{
									green_val-=2;
									if (green_val < 0) green_val = 0;
								}
								if (key==3)
								{
									blue_val-=2;
									if (blue_val < 0) blue_val = 0;
								}

								break;}    
		    		case 32: { // Ch + in Abhngigkeit Zifferntaste
								if (key==1)
								{
									red_val+=8;
									if (red_val > 63) red_val = 63;
								}
								if (key==2)
								{
									green_val+=8;
									if (green_val > 63) green_val = 63;
								}
								if (key==3)
								{
									blue_val+=8;
									if (blue_val > 63) blue_val = 63;
								}
								break;
							}    
		    		case 33: { // Ch - in Abhngigkeit Zifferntaste
								if (key==1)
								{
									red_val-=8;
									if (red_val < 0) red_val = 0;
								}
								if (key==2)
								{
									green_val-=8;
									if (green_val < 0) green_val = 0;
								}
								if (key==3)
								{
									blue_val-=8;
									if (blue_val < 0) blue_val = 0;
								}

								break;}    

					case 1:	{	key=1; break;}		//Farbe1
					case 2:	{	key=2; break;}		//Farbe 2
					case 3:	{	key=3; break;}		//Farbe 3
					case 5:	{	key=5; break;}		//Speichern einleiten
					case 6:	{	iProg=6; break;}	//Prog HSB
					case 9: {	iProg=9; break;}	//Prog RGB / bri
					case 0: {	iProg=0; break;}	//Prog RGB
		    		default: ;    

				  } //switch (code)

			  } //addr==0
      

		   } //FB-Verarbeitung


			//HSB-Werte aus Potis interpretieren (Taste 6 an FB gedrckt)
			if (iProg==6)
			{		   
				if (hue > 764) hue = 0;

				if (hue < 255) {
				   red_val = (10880 - sat * (hue - 170)) >> 7;
				   green_val = (10880 - sat * (85 - hue)) >> 7;
				   blue_val = (10880 - sat * 85) >> 7;
				}
				else if (hue < 510) {
				   red_val = (10880 - sat * 85) >> 7;
				   green_val = (10880 - sat * (hue - 425)) >> 7;
				   blue_val = (10880 - sat * (340 - hue)) >> 7;
				}
				else {
				   red_val = (10880 - sat * (595 - hue)) >> 7;
				   green_val = (10880 - sat * 85) >> 7;
				   blue_val = (10880 - sat * (hue - 680)) >> 7;
				}
				//im HSB-Programm geht Helligkeit bis 255 (*4)
				red = (((bri*4) + 1) * red_val) >> 8;
				green = (((bri*4) + 1) * green_val) >> 8;
				blue = (((bri*4) + 1) * blue_val) >> 8;		
				OCR1A = red;
				OCR1B = green;
				OCR2 = blue;		
			}
			else if (iProg==0)	//RGB-Werte verarbeiten Prog 0
			{
				//Helligkeit Multiplizieren fr Prog 0
				red = ((bri + 1) * red_val) >> 6;
				green = ((bri + 1) * green_val) >> 6;
				blue = ((bri + 1) * blue_val) >> 6;		   
				pwm_10_64(red, 1);
				pwm_10_64(green, 2);
				pwm_10_64(blue, 3);
			}
			else if (iProg==9)	//ohne Helligkeitsregelung
			{
				//direkte Ausgabe der Werte
				pwm_10_64(red_val, 1);
				pwm_10_64(green_val, 2);
				pwm_10_64(blue_val, 3);
			}
		//}
	  	//else Fernbedienung


   }    //while                     
 
   /* wird nie erreicht */
   return 0;                 
}
